@@ -29,6 +29,10 @@
 
 #include <glib.h>
 
+#include <fast_float/fast_float.h>
+
+#include <fmt/format.h>
+
 #include "color-names.hh"
 
 namespace vte::color::impl {
@@ -298,7 +302,7 @@ parse_csslike(std::string const& spec) noexcept
                 auto value = uint64_t{};
                 auto const start = spec.c_str() + 1;
                 auto const end = spec.c_str() + spec.size();
-                auto const rv = std::from_chars(start, end, value, 16);
+                auto const rv = fast_float::from_chars(start, end, value, 16);
                 if (rv.ec != std::errc{} || rv.ptr != end)
                         return std::nullopt;
 
@@ -424,7 +428,7 @@ parse_x11like(std::string const& spec) noexcept
                 auto value = uint64_t{};
                 auto const start = spec.c_str() + 1;
                 auto const end = spec.c_str() + spec.size();
-                auto const rv = std::from_chars(start, end, value, 16);
+                auto const rv = fast_float::from_chars(start, end, value, 16);
                 if (rv.ec != std::errc{} || rv.ptr != end)
                         return std::nullopt;
 
@@ -447,13 +451,13 @@ parse_x11like(std::string const& spec) noexcept
                 // Note that the length check above makes sure that @r, @g, @b,
                 // don't exceed @bits.
                 auto r = UINT64_C(0), b = UINT64_C(0), g = UINT64_C(0);
-                auto rv = std::from_chars(start, end, r, 16);
+                auto rv = fast_float::from_chars(start, end, r, 16);
                 if (rv.ec != std::errc{} || rv.ptr == end || *rv.ptr != '/')
                         return std::nullopt;
-                rv = std::from_chars(rv.ptr + 1, end, g, 16);
+                rv = fast_float::from_chars(rv.ptr + 1, end, g, 16);
                 if (rv.ec != std::errc{} || rv.ptr == end || *rv.ptr != '/')
                         return std::nullopt;
-                rv = std::from_chars(rv.ptr + 1, end, b, 16);
+                rv = fast_float::from_chars(rv.ptr + 1, end, b, 16);
                 if (rv.ec != std::errc{} || rv.ptr != end)
                         return std::nullopt;
 
@@ -506,14 +510,10 @@ to_string(color_tuple const& tuple,
 {
         switch (fmt) {
                 using enum color_output_format;
-        case HEX: {
-                char buf[32];
-                auto const len = g_snprintf(buf,
-                                            sizeof(buf),
-                                            alpha ? "#%08X" : "#%06X",
-                                            unsigned(impl::to_bits(tuple, 8, alpha)));
-                return {buf, size_t(len)};
-        }
+        case HEX:
+                return fmt::format("#{:0{}X}",
+                                   unsigned(impl::to_bits(tuple, 8, alpha)),
+                                   alpha ? 8 : 6);
 
         default:
                 __builtin_unreachable();
