@@ -1,15 +1,15 @@
 /*
  * Copyright © 2017, 2018, 2019 Christian Persch
  *
- * This programme is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public
- * License as published by the Free Software Foundation; either
- * version 3 of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- * This programme is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <https://www.gnu.org/licenses/>.
@@ -31,11 +31,14 @@
 
 #include <string>
 
-#include "debug.h"
+#include <fmt/format.h>
+
+#include "debug.hh"
 #include "glib-glue.hh"
+#include "libc-glue.hh"
 #include "utf8.hh"
 
-#ifdef WITH_ICU
+#if WITH_ICU
 #include "icu-decoder.hh"
 #include "icu-glue.hh"
 #endif
@@ -50,98 +53,80 @@ private:
         bool m_quiet{false};
         bool m_statistics{false};
         bool m_utf8{false};
+        int m_buffer_size{16384};
         int m_repeat{1};
-        char* m_charset{nullptr};
-        char** m_filenames{nullptr};
-
-        template<typename T1, typename T2 = T1>
-        class OptionArg {
-        private:
-                T1* m_return_ptr;
-                T2 m_value;
-        public:
-                OptionArg(T1* ptr, T2 v) : m_return_ptr{ptr}, m_value{v} { }
-                ~OptionArg() { *m_return_ptr = m_value; }
-
-                inline constexpr T2* ptr() noexcept { return &m_value; }
-        };
-
-        using BoolArg = OptionArg<bool, gboolean>;
-        using IntArg = OptionArg<int>;
-        using StrArg = OptionArg<char*>;
-        using StrvArg = OptionArg<char**>;
+        vte::glib::StringPtr m_charset{};
+        vte::glib::StrvPtr m_filenames{};
 
 public:
 
         Options() noexcept = default;
         Options(Options const&) = delete;
         Options(Options&&) = delete;
-
-        ~Options() {
-                if (m_filenames != nullptr)
-                        g_strfreev(m_filenames);
-        }
-
         Options& operator=(Options const&) = delete;
         Options& operator=(Options&&) = delete;
+        ~Options() = default;
 
-        inline constexpr bool benchmark()  const noexcept { return m_benchmark;  }
-        inline constexpr bool codepoints() const noexcept { return m_codepoints; }
-        inline constexpr bool list()       const noexcept { return m_list;       }
-        inline constexpr bool statistics() const noexcept { return m_statistics; }
-        inline constexpr int  quiet()      const noexcept { return m_quiet;      }
-        inline constexpr bool utf8()       const noexcept { return m_utf8;       }
-        inline constexpr int  repeat()     const noexcept { return m_repeat;     }
-        inline constexpr char const* charset()          const noexcept { return m_charset;   }
-        inline constexpr char const* const* filenames() const noexcept { return m_filenames; }
+        inline constexpr bool   benchmark()   const noexcept { return m_benchmark;   }
+        inline constexpr size_t buffer_size() const noexcept { return std::max(m_buffer_size, 1); }
+        inline constexpr bool   codepoints()  const noexcept { return m_codepoints;  }
+        inline constexpr bool   list()        const noexcept { return m_list;        }
+        inline constexpr bool   statistics()  const noexcept { return m_statistics;  }
+        inline constexpr int    quiet()       const noexcept { return m_quiet;       }
+        inline constexpr bool   utf8()        const noexcept { return m_utf8;        }
+        inline constexpr int    repeat()      const noexcept { return m_repeat;      }
+        inline char const* charset()          const noexcept { return m_charset.get();   }
+        inline char const* const* filenames() const noexcept { return m_filenames.get(); }
 
         bool parse(int argc,
                    char* argv[],
                    GError** error) noexcept
         {
-                {
-                        auto benchmark = BoolArg{&m_benchmark, false};
-                        auto codepoints = BoolArg{&m_codepoints, false};
-                        auto list = BoolArg{&m_list, false};
-                        auto quiet = BoolArg{&m_quiet, false};
-                        auto statistics = BoolArg{&m_statistics, false};
-                        auto utf8 = BoolArg{&m_utf8, false};
-                        auto repeat = IntArg{&m_repeat, 1};
-                        auto charset = StrArg{&m_charset, nullptr};
-                        auto filenames = StrvArg{&m_filenames, nullptr};
-                        GOptionEntry const entries[] = {
-                                { "benchmark", 'b', 0, G_OPTION_ARG_NONE, benchmark.ptr(),
-                                  "Measure time spent parsing each file", nullptr },
-                                { "codepoints", 'u', 0, G_OPTION_ARG_NONE, codepoints.ptr(),
-                                  "Output unicode code points by number", nullptr },
-                                { "charset", 'f', 0, G_OPTION_ARG_STRING, charset.ptr(),
-                                  "Input charset", "CHARSET" },
-                                { "list-charsets", 'l', 0, G_OPTION_ARG_NONE, list.ptr(),
-                                  "List available charsets", nullptr },
-                                { "quiet", 'q', 0, G_OPTION_ARG_NONE, quiet.ptr(),
-                                  "Suppress output except for statistics and benchmark", nullptr },
-                                { "repeat", 'r', 0, G_OPTION_ARG_INT, repeat.ptr(),
-                                  "Repeat each file COUNT times", "COUNT" },
-                                { "statistics", 's', 0, G_OPTION_ARG_NONE, statistics.ptr(),
-                                  "Output statistics", nullptr },
-                                { "utf-8", '8', 0, G_OPTION_ARG_NONE, utf8.ptr(),
-                                  "UTF-8 input (default)", nullptr },
-                                { G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_FILENAME_ARRAY, filenames.ptr(),
-                                  nullptr, nullptr },
-                                { nullptr },
-                        };
+                using BoolOption = vte::ValueGetter<bool, gboolean>;
+                using IntOption = vte::ValueGetter<int, int>;
+                using StringOption = vte::ValueGetter<vte::glib::StringPtr, char*, nullptr>;
+                using StrvOption = vte::ValueGetter<vte::glib::StrvPtr, char**, nullptr>;
 
-                        auto context = g_option_context_new("[FILE…] — decoder cat");
-                        g_option_context_set_help_enabled(context, true);
-                        g_option_context_add_main_entries(context, entries, nullptr);
+                auto benchmark = BoolOption{m_benchmark, false};
+                auto codepoints = BoolOption{m_codepoints, false};
+                auto list = BoolOption{m_list, false};
+                auto quiet = BoolOption{m_quiet, false};
+                auto statistics = BoolOption{m_statistics, false};
+                auto utf8 = BoolOption{m_utf8, false};
+                auto buffer_size = IntOption{m_buffer_size, 16384};
+                auto repeat = IntOption{m_repeat, 1};
+                auto charset = StringOption{m_charset, nullptr};
+                auto filenames = StrvOption{m_filenames, nullptr};
 
-                        auto rv = bool{g_option_context_parse(context, &argc, &argv, error) != false};
-                        g_option_context_free(context);
-                        if (!rv)
-                                return rv;
-                }
+                GOptionEntry const entries[] = {
+                        { "benchmark", 'b', 0, G_OPTION_ARG_NONE, &benchmark,
+                          "Measure time spent parsing each file", nullptr },
+                        { "buffer-size", 'B', 0, G_OPTION_ARG_INT, &buffer_size,
+                          "Buffer size", "SIZE" },
+                        { "codepoints", 'u', 0, G_OPTION_ARG_NONE, &codepoints,
+                          "Output unicode code points by number", nullptr },
+                        { "charset", 'f', 0, G_OPTION_ARG_STRING, &charset,
+                          "Input charset", "CHARSET" },
+                        { "list-charsets", 'l', 0, G_OPTION_ARG_NONE, &list,
+                          "List available charsets", nullptr },
+                        { "quiet", 'q', 0, G_OPTION_ARG_NONE, &quiet,
+                          "Suppress output except for statistics and benchmark", nullptr },
+                        { "repeat", 'r', 0, G_OPTION_ARG_INT, &repeat,
+                          "Repeat each file COUNT times", "COUNT" },
+                        { "statistics", 's', 0, G_OPTION_ARG_NONE, &statistics,
+                          "Output statistics", nullptr },
+                        { "utf-8", '8', 0, G_OPTION_ARG_NONE, &utf8,
+                          "UTF-8 input (default)", nullptr },
+                        { G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_FILENAME_ARRAY, &filenames,
+                          nullptr, nullptr },
+                        { nullptr },
+                };
 
-                return true;
+                auto context = vte::take_freeable(g_option_context_new("[FILE…] — decoder cat"));
+                g_option_context_set_help_enabled(context.get(), true);
+                g_option_context_add_main_entries(context.get(), entries, nullptr);
+
+                return g_option_context_parse(context.get(), &argc, &argv, error);
         }
 }; // class Options
 
@@ -150,27 +135,6 @@ private:
         std::string m_str{};
         bool m_codepoints{false};
 
-        void
-        print(char const* buf,
-              size_t len) noexcept
-        {
-                m_str.append(buf, len);
-        }
-
-        G_GNUC_PRINTF(2, 3)
-        void
-        print_format(char const* format,
-                     ...)
-        {
-                char buf[256];
-                va_list args;
-                va_start(args, format);
-                auto const len = g_vsnprintf(buf, sizeof(buf), format, args);
-                va_end(args);
-
-                m_str.append(buf, len);
-        }
-
         void
         print_u32(uint32_t const c) noexcept
         {
@@ -180,12 +144,14 @@ private:
                 if (m_codepoints) {
                         ubuf[len] = 0;
                         if (g_unichar_isprint(c)) {
-                                print_format("[%04X %s]", c, ubuf);
+                                fmt::format_to(std::back_inserter(m_str),
+                                               "<U+{:04X} '{}'>", c, ubuf);
                         } else {
-                                print_format("[%04X]", c);
+                                fmt::format_to(std::back_inserter(m_str),
+                                               "<U+{:04X}>", c);
                         }
                 } else {
-                        print(ubuf, len);
+                        m_str.append(ubuf, len);
                 }
         }
 
@@ -231,7 +197,7 @@ public:
 
 }; // class Sink
 
-#ifdef WITH_ICU
+#if WITH_ICU
 
 static std::unique_ptr<vte::base::ICUDecoder>
 make_decoder(Options const& options)
@@ -240,26 +206,32 @@ make_decoder(Options const& options)
 
         auto converter = std::shared_ptr<UConverter>{ucnv_open(options.charset(), err), &ucnv_close};
         if (err.isFailure()) {
-                if (!options.quiet())
-                        g_printerr("Failure to open converter for \"%s\": %s\n",
-                                   options.charset(), err.errorName());
+                if (!options.quiet()) {
+                        fmt::println(stderr,
+                                     "Failure to open converter for \"{}\": {}",
+                                     options.charset(), err.errorName());
                 return {};
+                }
         }
 
         if (err.get() == U_AMBIGUOUS_ALIAS_WARNING) {
                 err.reset();
                 auto canonical = ucnv_getName(converter.get(), err);
-                if (err.isSuccess() && !options.quiet())
-                        g_printerr("Warning: charset \"%s\" is ambigous alias for \"%s\"\n",
-                                   options.charset(), canonical);
+                if (err.isSuccess() && !options.quiet()) {
+                        fmt::println(stderr,
+                                     "Warning: charset \"{}\" is ambigous alias for \"{}\"",
+                                     options.charset(), canonical);
+                }
         }
 
         err.reset();
         auto u32_converter = std::shared_ptr<UConverter>{ucnv_open("utf32platformendian", err), &ucnv_close};
         if (err.isFailure()) {
-                if (!options.quiet())
-                        g_printerr("Failure to open converter for \"%s\": %s\n",
-                                   "UTF-32", err.errorName());
+                if (!options.quiet()) {
+                        fmt::println(stderr,
+                                     "Failure to open converter for \"{}\": {}",
+                                     "UTF-32", err.errorName());
+                }
                 return {};
         }
 
@@ -278,18 +250,19 @@ private:
         template<class Functor>
         void
         process_file_utf8(int fd,
+                          Options const& options,
                           Functor& func)
         {
                 auto decoder = vte::base::UTF8Decoder{};
 
-                auto const buf_size = size_t{16384};
-                auto buf = g_new0(uint8_t, buf_size);
+                auto const buffer_size = options.buffer_size();
+                auto buf = g_new0(uint8_t, buffer_size);
 
                 auto start_time = g_get_monotonic_time();
 
                 auto buf_start = size_t{0};
                 for (;;) {
-                        auto len = read(fd, buf + buf_start, buf_size - buf_start);
+                        auto len = read(fd, buf + buf_start, buffer_size - buf_start);
                         if (!len)
                                 break;
                         if (len == -1) {
@@ -336,23 +309,24 @@ private:
                 g_free(buf);
         }
 
-#ifdef WITH_ICU
+#if WITH_ICU
         template<class Functor>
         void
         process_file_icu(int fd,
+                         Options const& options,
                          vte::base::ICUDecoder* decoder,
                          Functor& func)
         {
                 decoder->reset();
 
-                auto const buf_size = size_t{16384};
-                auto buf = g_new0(uint8_t, buf_size);
+                auto const buffer_size = options.buffer_size();
+                auto buf = g_new0(uint8_t, buffer_size);
 
                 auto start_time = g_get_monotonic_time();
 
                 auto buf_start = size_t{0};
                 while (true) {
-                        auto len = read(fd, buf + buf_start, buf_size - buf_start);
+                        auto len = read(fd, buf + buf_start, buffer_size - buf_start);
                         if (!len) /* EOF */
                                 break;
                         if (len == -1) {
@@ -389,7 +363,7 @@ private:
                 }
 
                 /* Flush remaining output */
-                auto sptr = reinterpret_cast<uint8_t const*>(buf + buf_size);
+                auto sptr = reinterpret_cast<uint8_t const*>(buf + buffer_size);
                 auto result = vte::base::ICUDecoder::Result{};
                 while ((result = decoder->decode(&sptr, true)) == vte::base::ICUDecoder::Result::eSomething) {
                         func(decoder->codepoint());
@@ -409,7 +383,7 @@ private:
                      Options const& options,
                      Functor& func)
         {
-#ifdef WITH_ICU
+#if WITH_ICU
                 auto decoder = std::unique_ptr<vte::base::ICUDecoder>{};
                 if (options.charset()) {
                         decoder = make_decoder(options);
@@ -422,17 +396,19 @@ private:
 
                 for (auto i = 0; i < options.repeat(); ++i) {
                         if (i > 0 && lseek(fd, 0, SEEK_SET) != 0) {
-                                g_printerr("Failed to seek: %m\n");
+                                auto errsv = vte::libc::ErrnoSaver{};
+                                fmt::println(stderr,
+                                             "Failed to seek: {}", g_strerror(errsv));
                                 return false;
                         }
 
-#ifdef WITH_ICU
+#if WITH_ICU
                         if (decoder) {
-                                process_file_icu(fd, decoder.get(), func);
+                                process_file_icu(fd, options, decoder.get(), func);
                         } else
 #endif
                         {
-                                process_file_utf8(fd, func);
+                                process_file_utf8(fd, options, func);
                         }
                 }
 
@@ -466,13 +442,17 @@ public:
                                         fd = STDIN_FILENO;
 
                                         if (options.repeat() != 1) {
-                                                g_printerr("Cannot consume STDIN more than once\n");
+                                                fmt::println(stderr,
+                                                             "Cannot consume STDIN more than once");
                                                 return false;
                                         }
                                 } else {
                                         fd = ::open(filename, O_RDONLY);
                                         if (fd == -1) {
-                                                g_printerr("Error opening file %s: %m\n", filename);
+                                                auto errsv = vte::libc::ErrnoSaver{};
+                                                fmt::println(stderr,
+                                                             "Error opening file \"{}\": {}",
+                                                             filename, g_strerror(errsv));
                                         }
                                 }
                                 if (fd != -1) {
@@ -492,9 +472,9 @@ public:
 
         void print_statistics() const noexcept
         {
-                g_printerr("%\'16" G_GSIZE_FORMAT " input bytes produced %\'16" G_GSIZE_FORMAT
-                           " unichars and %" G_GSIZE_FORMAT " errors\n",
-                           m_input_bytes, m_output_chars, m_errors);
+                fmt::println(stderr,
+                             "{} input bytes produced {} unichars and {} errors",
+                             m_input_bytes, m_output_chars, m_errors);
         }
 
         void print_benchmark() const noexcept
@@ -510,15 +490,16 @@ public:
                 for (unsigned int i = 0; i < m_bench_times->len; ++i)
                         total_time += g_array_index(m_bench_times, int64_t, i);
 
-                g_printerr("\nTimes: best %\'" G_GINT64_FORMAT "µs "
-                           "worst %\'" G_GINT64_FORMAT "µs "
-                           "average %\'" G_GINT64_FORMAT "µs\n",
-                           g_array_index(m_bench_times, int64_t, 0),
-                           g_array_index(m_bench_times, int64_t, m_bench_times->len - 1),
-                           total_time / (int64_t)m_bench_times->len);
-                for (unsigned int i = 0; i < m_bench_times->len; ++i)
-                        g_printerr("  %\'" G_GINT64_FORMAT "µs\n",
-                                   g_array_index(m_bench_times, int64_t, i));
+                fmt::println(stderr,
+                             "\nTimes: best {}µs worst {}µs average {}µs",
+                             g_array_index(m_bench_times, int64_t, 0),
+                             g_array_index(m_bench_times, int64_t, m_bench_times->len - 1),
+                             total_time / (int64_t)m_bench_times->len);
+                for (unsigned int i = 0; i < m_bench_times->len; ++i) {
+                        fmt::println(stderr,
+                                     "  {:>10}µs",
+                                     g_array_index(m_bench_times, int64_t, i));
+                }
         }
 
 }; // class Processor
@@ -535,20 +516,23 @@ main(int argc,
         auto options = Options{};
         auto error = vte::glib::Error{};
         if (!options.parse(argc, argv, error)) {
-                g_printerr("Failed to parse arguments: %s\n", error.message());
+                fmt::println(stderr,
+                             "Failed to parse arguments: {}",
+                             error.message());
                 return EXIT_FAILURE;
         }
 
         if (options.list()) {
-#ifdef WITH_ICU
+#if WITH_ICU
                 auto charsets = vte::base::get_icu_charsets(true);
-                for (auto i = 0; charsets[i]; ++i)
-                        g_print("%s\n", charsets[i]);
+                for (auto i = 0; charsets[i]; ++i) {
+                        fmt::println(stdout, "{}", charsets[i]);
+                }
                 g_strfreev(charsets);
 
                 return EXIT_SUCCESS;
 #else
-                g_printerr("ICU support not available.\n");
+                fmt::println(stderr, "ICU support not available");
                 return EXIT_FAILURE;
 #endif
         }
