@@ -1,18 +1,18 @@
 /*
  * Copyright © 2019 Christian Persch
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 3 of the License, or (at your option) any later version.
+ * This library is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as published
+ * by the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <https://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this library.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -24,7 +24,7 @@
 
 #include <unicode/errorcode.h>
 
-#include "debug.h"
+#include "debug.hh"
 #include "icu-glue.hh"
 
 namespace vte::base {
@@ -71,8 +71,8 @@ ICUConverter::convert(std::string_view const& data)
                                       data.data(), data.size(),
                                       err);
         if (err.isFailure() && (err.get() != U_BUFFER_OVERFLOW_ERROR)) {
-                _vte_debug_print(VTE_DEBUG_CONVERSION,
-                                 "Error converting from UTF-8 to UTF-16 in preflight: %s\n",
+                _vte_debug_print(vte::debug::category::CONVERSION,
+                                 "Error converting from UTF-8 to UTF-16 in preflight: {}",
                                  err.errorName());
                 return {};
         }
@@ -90,8 +90,8 @@ ICUConverter::convert(std::string_view const& data)
                                  data.size(),
                                  err);
         if (err.isFailure()) {
-                _vte_debug_print(VTE_DEBUG_CONVERSION,
-                                 "Error converting from UTF-8 to UTF-16: %s\n",
+                _vte_debug_print(vte::debug::category::CONVERSION,
+                                 "Error converting from UTF-8 to UTF-16: {}",
                                  err.errorName());
                 return {};
         }
@@ -105,8 +105,8 @@ ICUConverter::convert(std::string_view const& data)
                                            u16_size,
                                            err);
         if (err.isFailure() && (err.get() != U_BUFFER_OVERFLOW_ERROR)) {
-                _vte_debug_print(VTE_DEBUG_CONVERSION,
-                                 "Error converting from UTF-8 to %s in preflight: %s\n",
+                _vte_debug_print(vte::debug::category::CONVERSION,
+                                 "Error converting from UTF-8 to {} in preflight: {}",
                                  m_charset.c_str(),
                                  err.errorName());
                 return {};
@@ -125,8 +125,8 @@ ICUConverter::convert(std::string_view const& data)
                                       u16_size,
                                       err);
         if (err.isFailure()) {
-                _vte_debug_print(VTE_DEBUG_CONVERSION,
-                                 "Error converting from UTF-16 to %s: %s\n",
+                _vte_debug_print(vte::debug::category::CONVERSION,
+                                 "Error converting from UTF-16 to {}: {}",
                                  m_charset.c_str(),
                                  err.errorName());
                 return {};
